import tkinter as tk
from pyrustic.viewable import Viewable
from pyrustic.widget.toast import Toast
from pyrustic.widget.choice import Choice
from hubway.view.auth_view import AuthView
from hubway.view.publishing_view import PublishingView


class FooterView(Viewable):
    def __init__(self, master, main_view, main_host):
        self._master = master
        self._main_view = main_view
        self._main_host = main_host
        self._body = None
        self._auth_view = None
        self._auth_strvar = tk.StringVar(value="Authenticate")
        self._auth_intvar = tk.IntVar(value=0)
        self._cache = None
        self._toast_auth = None

    # ===================================
    #              PUBLIC
    # ===================================
    def set_auth(self, code, info, data):
        if code in (200, 304):
            text = "Authenticated ({})".format(data)
            self._cache = (1, text)
            self._auth_strvar.set(text)
            self._auth_intvar.set(1)
            welcome_text = "Welcome Friend"
            if code == 304:
                welcome_text = "Welcome back Friend !"
            Toast(message=welcome_text).build()
        else:
            Toast(message=info).build()
            self.cancel_authenticating()
        if self._toast_auth:
            self._toast_auth.destroy()
            self._toast_auth = None

    def notify_authenticating(self):
        message = "Authenticating..."
        self._toast_auth = Toast(self._body, message=message, duration=None)
        self._toast_auth.build()

    def cancel_authenticating(self):
        self._auth_intvar.set(0)
        self._auth_strvar.set("Authenticate")

    # ===================================
    #              LIFECYCLE
    # ===================================
    def _on_build(self):
        self._body = tk.Frame(self._master)
        # checkbutton Auth
        checkbutton_auth = tk.Checkbutton(self._body,
                                          name="checkbutton_auth",
                                          textvariable=self._auth_strvar,
                                          variable=self._auth_intvar,
                                          onvalue=1, offvalue=0,
                                          command=self._on_click_checkbutton)
        checkbutton_auth.pack(side=tk.LEFT, anchor="s", ipadx=5)
        # button Publish
        button_publishing = tk.Button(self._body, name="button_publishing", text="Publish",
                                    command=self._on_click_button_publishing)
        button_publishing.pack(side=tk.RIGHT, padx=2, pady=2)

    def _on_display(self):
        pass

    def _on_destroy(self):
        pass

    # =============================
    #           PRIVATE
    # =============================
    def _on_click_checkbutton(self):
        state = self._auth_intvar.get()
        text = self._auth_strvar.get()
        if state == 0:
            self._auth_strvar.set("Authenticate")
            self._auth_intvar.set(0)
            self._main_host.unauth()
        else:
            self._auth_strvar.set("Authenticating...")
            auth_view = AuthView(self._master, self._main_view, self._main_host)
            auth_view.build_wait()

    def _on_click_button_publishing(self):
        if self._main_host.login is None:
            toast = Toast(self._body, message="Please authenticate yourself !")
            toast.build()
            return
        items = self._main_host.get_assets_from_dist_folder()
        if not items:
            message = "Please create an asset first !\nType 'pkg' in the Manager."
            toast = Toast(self._body, message=message,
                          duration=5000)
            toast.build_wait()
            return
        message = "This is the list of available assets in\n"
        message += "the folder $TARGET/pyrustic_data/dist"
        choice = Choice(self._body,
                        items=items,
                        title="Asset selection",
                        header="Select an asset",
                        message=message,
                        flavor="radio")
        choice.build_wait()
        asset_version = choice.selected
        if not asset_version:
            return
        asset_version = asset_version[1]
        publishing_view = PublishingView(self._body,
                                         self._main_view,
                                         self._main_host,
                                         asset_version)
        publishing_view.build()

    def _get_scrollbox_config(self):
        data = { "scrollbox":
                    {"canvas":
                         {"width": 100}}}
        return data



import tkinter as tk
from pyrustic.threadom import Threadom
from pyrustic.viewable import Viewable
from hubway.host.main_host import MainHost
from hubway.view.header_view import HeaderView
from hubway.view.central_view import CentralView
from hubway.view.footer_view import FooterView


class MainView(Viewable):
    def __init__(self, app):
        self._app = app
        self._root = app.root
        self._main_host = MainHost()
        self._threadom = Threadom(self._root)
        self._body = None
        self._header_view = None
        self._central_view = None
        self._footer_view = None

    @property
    def root(self):
        return self._root

    @property
    def threadom(self):
        return self._threadom

    @property
    def header_view(self):
        return self._header_view

    @property
    def central_view(self):
        return self._central_view

    @property
    def footer_view(self):
        return self._footer_view

    def _on_build(self):
        self._body = tk.Frame(self._root)
        # header
        self._header_view = HeaderView(self._body, self, self._main_host)
        self._header_view.build_pack(fill=tk.X, pady=(0, 10))
        # central
        self._central_view = CentralView(self._body, self, self._main_host)
        self._central_view.build_pack(expand=1, fill=tk.BOTH, padx=(5, 3))
        # footer
        self._footer_view = FooterView(self._body, self, self._main_host)
        self._footer_view.build_pack(fill=tk.X, pady=(10, 0))

    def _on_display(self):
        pass

    def _on_destroy(self):
        pass



import os.path
from pyrustic.app import App
from hubway import about
from hubway.misc import my_theme
from hubway.view.main_view import MainView


def main():
    app = App(about.ROOT_DIR)
    app.root.title("Hubway - built with Pyrustic")
    app.theme = my_theme.get_theme()
    app.view = MainView(app)
    app.center()
    app.start()


if __name__ == "__main__":
    main()



from pyrustic.manager.misc import funcs
from pyrustic.manager import constant
from pyrustic.jasonix import Jasonix
from hubway.host.publishing_host import PublishingHost
from distutils.version import StrictVersion
import os
import os.path


class MainHost:
    def __init__(self):
        self._login = None
        self._jasonix = Jasonix(constant.HUB_SHARED_DATA_FILE,
                                default=constant.DEFAULT_HUB_SHARED_DATA_FILE)
        self._gurl = funcs.create_gurl()

    @property
    def login(self):
        return self._login

    def auth(self, token):
        """
        Return: status_code, status_text, data
        data = the login if status is 200 or 304, else data is None
        """
        self._gurl.token = token
        res = "/user"
        response = self._gurl.request(funcs.get_hub_url(res))
        code = response.code
        json = response.json
        data = None
        if code == 304:
            json = response.cached_response.json
        if (code in (200, 304)) and json:
            data = json["login"]
        else:
            self._gurl.token = None
        self._login = data
        return (*response.status, data)

    def unauth(self):
        self._login = None
        self._gurl.token = None

    def rate(self):
        """
        Return: status_code, status_text, data
        data = {"limit": int, "remaining": int}
        """
        res = "/rate_limit"
        response = self._gurl.request(funcs.get_hub_url(res))
        code = response.code
        json = response.json
        data = {}
        if code == 304:
            json = response.cached_response.json
        if (code in (200, 304)) and json:
            data["limit"] = json["resources"]["core"]["limit"]
            data["remaining"] = json["resources"]["core"]["remaining"]
        return (*response.status, data)

    def repo_description(self, owner, repo):
        """
        Returns: (status_code, status_text, data)
        data = {"created_at": date, "description": str,
                "stargazers_count": int, "subscribers_count": int}
        """
        res = "/repos/{}/{}".format(owner, repo)
        response = self._gurl.request(funcs.get_hub_url(res))
        code = response.code
        json = response.json
        data = {}
        if code == 304:
            json = response.cached_response.json
        if (code in (200, 304)) and json:
            data["description"] = json["description"]
            date = json["created_at"]
            data["created_at"] = self._badass_iso_8601_date_parser(date)
            data["stargazers_count"] = json["stargazers_count"]
            data["subscribers_count"] = json["subscribers_count"]
        return (*response.status, data)

    def latest_release(self, owner, repo):
        """
        Returns: (status_code, status_text, data}
        data = {"tag_name": str, "published_at": date,
                "downloads_count": int}
        """
        res = "/repos/{}/{}/releases/latest".format(owner, repo)
        response = self._gurl.request(funcs.get_hub_url(res))
        code = response.code
        json = response.json
        data = {}
        if code == 304:
            json = response.cached_response.json
        if (code in (200, 304)) and json:
            data["tag_name"] = json["tag_name"]
            date = json["published_at"]
            data["published_at"] = self._badass_iso_8601_date_parser(date)
            data["downloads_count"] = self._downloads_counter(json)
        return (*response.status, data)

    def latest_releases_downloads(self, owner, repo, maxi=10):
        """
        Returns: (status_code, status_text, data}
        data = int, downloads count
        """
        res = "/repos/{}/{}/releases?per_page={}".format(owner, repo, maxi)
        response = self._gurl.request(funcs.get_hub_url(res))
        code = response.code
        json = response.json
        data = 0
        if code == 304:
            json = response.cached_response.json
        if (code in (200, 304)) and json:
            for release in json:
                data += self._downloads_counter(release)
        return (*response.status, data)

    def get_assets_from_dist_folder(self):
        dist_folder = os.path.join(self.target_project(),
                                   "pyrustic_data", "dist")
        if not os.path.exists(dist_folder):
            return []
        versions = []
        for item in os.listdir(dist_folder):
            path = os.path.join(dist_folder, item)
            if not os.path.isfile(path):
                continue
            cache = os.path.splitext(item)
            if len(cache) != 2:
                continue
            version, ext = cache
            versions.append(version)
        versions.sort(key=StrictVersion)
        versions.reverse()
        return versions

    def publishing(self, owner, repo, name,
                   tag_name, target_commitish,
                   description, prerelease, draft,
                   asset_path, asset_name, asset_label):
        """
        Return meta code, status code, status text
        meta code:
            0: temporary copy and zip operation are success
            1: failed to copy the project in temp cache
            2: failed to zip the project
        """
        # exec tests
        # exec prolog
        # publishing
        # exec epilog
        publishing_host = PublishingHost(self._gurl, owner, repo)
        data = publishing_host.publishing(name, tag_name,
                                          target_commitish,
                                          description, prerelease,
                                          draft, asset_path,
                                          asset_name, asset_label)
        return data

    def update_activity(self, operation, owner, repo):
        if operation == "add":
            if owner in self._jasonix.data:
                if not repo in self._jasonix.data[owner]:
                    self._jasonix.data[owner].append(repo)
            else:
                self._jasonix.data[owner] = [repo]
        elif operation == "delete":
            if owner in self._jasonix.data:
                if repo is None:
                    del self._jasonix.data[owner]
                else:
                    for i, cache in enumerate(self._jasonix.data[owner]):
                        if cache == repo:
                            del self._jasonix.data[owner][i]
                            break
        self._jasonix.save()

    def last_activity(self):
        return self._jasonix.data

    def target_project(self):
        jasonix = funcs.get_manager_jasonix()
        target = jasonix.data["target"]
        return target

    def about_target_project(self):
        """  Returns a dict:
        {"target": str, "project_name": str, "version": str}
        """
        target = self.target_project()
        project_name = os.path.basename(target)
        version = "0.0.1"
        data = {"target": target,
                "project_name": project_name,
                "version": version}
        about_path = os.path.join(target, "pyrustic_data", "info.json")
        if not os.path.exists(about_path):
            return data
        jasonix = Jasonix(about_path)
        cache = jasonix.data.get("project_name", "")
        if cache:
            project_name = cache
        cache = jasonix.data.get("version", "")
        if cache:
            version = cache
        data["project_name"] = project_name
        data["version"] = version
        return data

    def extract_owner_repo_from_url(self, url):
        """ Returns a tuple: (owner, repo) """

        patterns_to_wipe = ("https://", "www.", "github.com")
        for pattern in patterns_to_wipe:
            url = url.replace(pattern, "", 1)
        url = url.lstrip("/")
        url = url.rstrip("/")
        splitted = url.split("/")
        if len(splitted) == 2:
            return splitted
        else:
            return None

    def _downloads_counter(self, json):
        count = 0
        for asset in json["assets"]:
            count += asset["download_count"]
        return count

    def _badass_iso_8601_date_parser(self, date):
        # YYYY-MM-DDTHH:MM:SSZ
        date = date.rstrip("Z")
        date_part, time_part = date.split("T")
        months = ("Jan", "Feb", "March", "April", "May", "June", "July",
                 "Aug", "Sept", "Oct", "Nov", "Dec")
        year, month, day = date_part.split("-")
        text = "{} {} {} at {}".format(day, months[int(month) - 1], year, time_part)
        return text

    def _exec_tests(self):
        return

    def _exec_prolog(self):
        return

    def _publishing(self):
        return

    def _exec_epilog(self):
        return



import tkinter as tk
from pyrustic.viewable import Viewable
from pyrustic.widget.scrollbox import Scrollbox
from hubway.view.tree_view import TreeView


class CentralView(Viewable):
    def __init__(self, master, main_view, main_host):
        self._master = master
        self._main_view = main_view
        self._main_host = main_host
        self._body = None
        self._tree_view = None

    @property
    def tree_view(self):
        return self._tree_view

    @property
    def scrollbox(self):
        return self._scrollbox

    # ===============================
    #            PROPERTY
    # ===============================

    # ===============================
    #            PUBLIC
    # ===============================

    def add_node(self, owner, repo):
        self._main_host.update_activity("add", owner, repo)
        self._collapse_nodes()
        self._tree_view.feed(datatype="add_owner_repo",
                             data=(owner, repo))

    # ===============================
    #            LIFECYCLE
    # ===============================
    def _on_build(self):
        self._body = tk.Frame(self._master)
        # scrollbox
        self._scrollbox = Scrollbox(self._body, orient="v")
        self._scrollbox.build_pack(expand=1, fill=tk.BOTH)
        # treeview
        self._tree_view = TreeView(self._scrollbox.box, self._main_view, self._main_host)
        self._tree_view.build_pack(expand=1, fill=tk.BOTH)
        # insert ghost root node
        self._insert_ghost_root()

    def _on_display(self):
        pass

    def _on_destroy(self):
        pass

    # ===============================
    #            PRIVATE
    # ===============================
    def _insert_ghost_root(self):
        # insert first node
        node_id = self._tree_view.insert()
        self._tree_view.ghost(node_id)

    def _collapse_nodes(self):
        for owner in self._tree_view.nodes:
            owner_node_id = owner["node_id"]
            if owner_node_id == 0:
                continue
            else:
                self._tree_view.collapse(owner_node_id)



import tkinter as tk
from pyrustic.viewable import Viewable
from pyrustic import tkmisc


class FailureView(Viewable):
    def __init__(self, master, main_view, main_host, data):
        self._master = master
        self._main_view = main_view
        self._main_host = main_host
        self._data = data
        self._body = None

    def _on_build(self):
        self._body = tk.Toplevel(self._master)
        self._body.title("Failure")
        # text
        text = tk.Text(self._body,
                       name="failure_view",
                       width=50, height=10)
        text.pack(padx=5, pady=5)
        text.insert("1.0", self._data)
        # button quit
        button = tk.Button(self._body, text="Close", command=self.destroy)
        button.pack(anchor="e", padx=2, pady=2)

    def _on_display(self):
        pass

    def _on_destroy(self):
        pass

    def _toplevel_geometry(self):
        super()._toplevel_geometry()
        tkmisc.dialog_effect(self._body)



from pyrustic.viewable import Viewable
import tkinter as tk


class NodeView(Viewable):
    def __init__(self, master, tree_view, main_view, main_host, node_id):
        self._master = master
        self._tree_view = tree_view
        self._main_view = main_view
        self._main_host = main_host
        self._node_id = node_id
        self._body = None
        self._strvar_expand = tk.StringVar()

    def edit_state(self, expanded=True):
        self._strvar_expand.set("-" if expanded else "+")

    def feed(self, datatype, data=None):
        if not self._node_still_exists():
            return
        status_code, status_text, data = data
        if status_code in (200, 304):
            success_layouts = {"description": self._set_description_layout,
                       "latest_release": self._set_latest_release_layout,
                       "total_downloads": self._set_total_downloads_layout}
            if datatype in success_layouts:
                success_layouts[datatype](data)
        else:
            self._set_failure_layout(datatype, status_code, status_text)

    def _on_build(self):
        self._body = tk.Frame(self._master)
        node = self._tree_view.node(self._node_id)
        data = node["data"]
        node_type = data["type"]
        if node_type in ("owner", "repo"):
            command = (lambda node_type=node_type:
                       self._on_click_button_expand(node_type))
            button_expand = tk.Button(self._body,
                                      name="button_expander",
                                      textvariable=self._strvar_expand,
                                      command=command)
            button_expand.pack(side=tk.LEFT, padx=2)
            instance_name = "label_owner" if node_type == "owner" else "label_repo"
            label = tk.Label(self._body, name=instance_name, text=data["name"])
            label.pack(side=tk.LEFT)
            command = self._on_click_button_close
            button_delete = tk.Button(self._body,
                                      name="button_close",
                                      text="x",
                                      command=command)
            button_delete.pack(side=tk.LEFT, padx=(10, 0))

    def _on_display(self):
        node = self._tree_view.node(self._node_id)
        expanded = node["expanded"]
        cache = "+"
        if expanded:
            cache = "-"
        self._strvar_expand.set(cache)

    def _on_destroy(self):
        pass

    def _on_click_button_expand(self, node_type):
        self._collexp_node()

    def _on_click_button_close(self):
        # get data from node
        node = self._tree_view.node(self._node_id)
        owner = None
        repo = None
        node_type = node["data"]["type"]
        node_name = node["data"]["name"]
        if node_type == "owner":
            owner = node_name
        else:
            repo = node_name
            parent_node = self._tree_view.node(node["parent"])
            owner = parent_node["data"]["name"]
        self._main_host.update_activity("delete", owner, repo)
        self._tree_view.delete(self._node_id)

    def _collexp_node(self):
        cache = "+"
        if self._strvar_expand.get() == "+":
            cache = "-"
        self._strvar_expand.set(cache)
        self._tree_view.collexp(self._node_id)

    def _set_loading_mode(self):
        node = self._tree_view.node(self._node_id)
        data = node["data"]
        self._clear_node()
        # label title
        label_title = tk.Label(self._body, name="label_info_title",
                               text="{}: ".format(data["name"]))
        label_title.pack(side=tk.LEFT, padx=0)
        # label info
        label_info = tk.Label(self._body, text="Loading...")
        label_info.pack(side=tk.LEFT, padx=0)

    def _clear_node(self):
        for child in self._body.winfo_children():
            child.destroy()

    def _set_description_layout(self, data):
        self._clear_node()
        # title
        text = self._tree_view.node(self._node_id)["data"]["name"]
        label_title = tk.Label(self._body, name="label_info_title", text=text)
        label_title.grid(column=0, row=0, sticky="w")
        # description
        text = data["description"]
        text = "- No description -" if text is None else text
        entry_description = tk.Entry(self._body, name="entry_repo_description",
                                     width=70)
        entry_description.insert(0, text)
        entry_description.config(state="readonly")
        entry_description.grid(column=0, row=1, sticky="w")
        # creation date
        text = "Created on {} UTC".format(data["created_at"])
        label_date = tk.Label(self._body, text=text)
        label_date.grid(column=0, row=2, sticky="w")
        # stargazers/subscribers
        stargazers_count = data["stargazers_count"]
        subscribers_count = data["subscribers_count"]
        suffix_str_stargazers = "s" if stargazers_count > 1 else ""
        suffix_str_subscribers = "s" if subscribers_count > 1 else ""
        text = "{} Stargazer{} and {} Subscriber{}".format(stargazers_count,
                                                           suffix_str_stargazers,
                                                           subscribers_count,
                                                           suffix_str_subscribers)
        label_stargazers = tk.Label(self._body, name="label_counts",
                                    text=text)
        label_stargazers.grid(column=0, row=3, sticky="w")
        # button refresh
        button_refresh = tk.Button(self._body, name="button_refresh",
                                   text="Refresh", command=self.populate)
        button_refresh.grid(column=0, row=4, sticky="w", pady=5)

    def _set_latest_release_layout(self, data):
        self._clear_node()
        # title
        text = self._tree_view.node(self._node_id)["data"]["name"]
        label_title = tk.Label(self._body, name="label_info_title", text=text)
        label_title.grid(column=0, row=0, sticky="w")
        # name
        text = "Tag name: {}".format(data["tag_name"])
        label_name = tk.Label(self._body, text=text)
        label_name.grid(column=0, row=1, sticky="w")
        # creation date
        text = "Published on {} UTC".format(data["published_at"])
        label_date = tk.Label(self._body, text=text)
        label_date.grid(column=0, row=2, sticky="w")
        # downloads
        downloads_count = data["downloads_count"]
        suffix_str_downloads = "s" if downloads_count > 1 else ""
        text = "{} Download{}".format(downloads_count, suffix_str_downloads)
        label_downloads = tk.Label(self._body, name="label_counts",
                                   text=text)
        label_downloads.grid(column=0, row=3, sticky="w")
        # button refresh
        button_refresh = tk.Button(self._body,
                                   name="button_refresh",
                                   text="Refresh", command=self.populate)
        button_refresh.grid(column=0, row=4, sticky="w", pady=5)

    def _set_total_downloads_layout(self, data):
        self._clear_node()
        # title
        text = self._tree_view.node(self._node_id)["data"]["name"]
        label_title = tk.Label(self._body, name="label_info_title", text=text)
        label_title.grid(column=0, row=0, sticky="w")
        # name
        suffix_str_downloads = "s" if data > 1 else ""
        text = "{} Download{}".format(data, suffix_str_downloads)
        label_downloads = tk.Label(self._body, name="label_counts",
                                   text=text)
        label_downloads.grid(column=0, row=1, sticky="w")
        # button refresh
        button_refresh = tk.Button(self._body, name="button_refresh",
                                   text="Refresh", command=self.populate)
        button_refresh.grid(column=0, row=2, sticky="w", pady=5)

    def _set_failure_layout(self, datatype, status_code, status_text):
        self._clear_node()
        name = self._tree_view.node(self._node_id)["data"]["name"]
        # frame
        frame_title = tk.Frame(self._body)
        frame_title.grid(column=0, row=0, sticky="w")
        # label title failure
        label_title = tk.Label(frame_title, name="label_info_title",
                               text="{}: ".format(name))
        label_title.pack(side=tk.LEFT, padx=0)
        # label info
        label_info = tk.Label(frame_title, name="label_error",
                              text=status_text)
        label_info.pack(side=tk.LEFT, padx=0)
        # button
        button_retry = tk.Button(self._body, name="button_retry",
                                 text="Retry", command=self.populate)
        button_retry.grid(column=0, row=1, sticky="w", pady=5)

    def populate(self):
        self._set_loading_mode()
        node = self._tree_view.node(self._node_id)
        node_type = node["data"]["type"]
        repo_node = self._tree_view.node(node["parent"])
        owner_node = self._tree_view.node(repo_node["parent"])
        repo = repo_node["data"]["name"]
        owner = owner_node["data"]["name"]
        threadom = self._main_view.threadom
        host_choice = {"description": self._main_host.repo_description,
                       "latest_release": self._main_host.latest_release,
                       "total_downloads": self._main_host.latest_releases_downloads}
        host = host_choice[node_type]
        host_args = (owner, repo)
        consumer = (lambda data, datatype=node_type,
                           self=self:
                    self.feed(datatype, data=data))
        threadom.run(host, args=host_args, consumer=consumer)

    def _node_still_exists(self):
        if self._tree_view.node(self._node_id) is None:
            return False
        return True



from pyrustic.viewable import Viewable
from pyrustic import tkmisc
import tkinter as tk


class AuthView(Viewable):
    def __init__(self, master, main_view, main_host):
        self._master = master
        self._main_view = main_view
        self._main_host = main_host
        self._body = None
        self._strvar_token = tk.StringVar()
        self._data_submitted = False

    def _on_build(self):
        self._body = tk.Toplevel(self._master)
        self._body.title("Authentication")
        # token
        label = tk.Label(self._body, text="Token")
        label.pack(anchor="w", padx=5, pady=(10, 0))
        entry = tk.Entry(self._body, show="*",
                         textvariable=self._strvar_token)
        entry.pack(anchor="w", fill=tk.X, padx=5, pady=(0, 30))
        entry.focus_set()
        entry.bind("<Return>", lambda e, self=self: self._on_click_auth())
        # footer
        footer = tk.Frame(self._body)
        footer.pack(padx=2, pady=2)
        # button auth
        button_auth = tk.Button(footer, name="button_confirm",
                                text="Authenticate",
                                command=self._on_click_auth)
        button_auth.pack(side=tk.RIGHT)
        # button cancel
        button_cancel = tk.Button(footer, name="button_cancel",
                                  text="Cancel",
                                  command=self._on_click_cancel)
        button_cancel.pack(side=tk.RIGHT, padx=(5, 2))

    def _on_display(self):
        tkmisc.dialog_effect(self._body)

    def _on_destroy(self):
        if not self._data_submitted:
            self._main_view.footer_view.cancel_authenticating()

    def _on_click_auth(self):
        token = self._strvar_token.get()
        if token:
            consumer = (lambda code, info, data:
                        self._main_view.footer_view.set_auth(code, info, data))
            self._main_view.threadom.run(self._main_host.auth, args=[token],
                                           consumer=consumer, unpack_result=True)
            self._data_submitted = True
            self._main_view.footer_view.notify_authenticating()
        self.destroy()

    def _on_click_cancel(self):
        self.destroy()



from pyrustic.viewable import Viewable
from pyrustic.widget.confirm import Confirm
from pyrustic.widget.toast import Toast
from pyrustic import tkmisc
import tkinter as tk
import os.path
from hubway.view.failure_view import FailureView


class PublishingView(Viewable):
    def __init__(self, master, main_view, main_host, asset_version):
        self._master = master
        self._main_view = main_view
        self._main_host = main_host
        self._asset_version = asset_version
        # stringvars and intvars
        self._strvar_target = tk.StringVar()
        self._strvar_owner = tk.StringVar()
        self._strvar_repo = tk.StringVar()
        self._strvar_target_commitish = tk.StringVar()
        self._strvar_tag_name = tk.StringVar()
        self._strvar_description = tk.StringVar()
        self._strvar_release_name = tk.StringVar()
        self._intvar_prerelease = tk.IntVar(value=0)
        self._intvar_draft = tk.IntVar(value=0)
        self._strvar_asset_name = tk.StringVar()
        self._strvar_asset_label = tk.StringVar()
        # cache toast "Processing..."
        self._cache_toast_processing = None
        # asset path
        self._asset_path = None
        # target project
        self._target_project = None

    def _on_build(self):
        self._body = tk.Toplevel()
        self._body.title("Publishing")
        self._body.resizable(0, 0)
        # header
        self._set_header(self._body)
        # frame_form
        frame_form = tk.Frame(self._body, name="frame_form")
        frame_form.pack(padx=10, pady=20, expand=1, fill=tk.BOTH)
        frame_form.columnconfigure(0, pad=5)
        frame_form.columnconfigure(1, pad=5)
        frame_form.rowconfigure(0, pad=10)
        frame_form.rowconfigure(1, pad=10)
        frame_form.rowconfigure(2, pad=10)
        frame_form.rowconfigure(3, pad=10)
        # owner
        self._set_widgets_owner(frame_form)
        # repo
        self._set_widgets_repo(frame_form)
        # target_commitish
        self._set_widgets_target_commitish(frame_form)
        # tag_name
        self._set_widgets_tag_name(frame_form)
        # release name
        self._set_widgets_release_name(frame_form)
        # archive format
        self._set_widgets_archive_format(frame_form)
        # asset_name
        self._set_widgets_asset_name(frame_form)
        # asset_label
        self._set_widgets_asset_label(frame_form)
        # description
        self._set_widgets_description(frame_form)
        # label to describe mandatory field
        self._set_widgets_describe_mandatory_fields(frame_form)
        # footer
        self._set_widgets_footer(self._body)

    def _on_display(self):
        self._populate()

    def _on_destroy(self):
        pass

    def _toplevel_geometry(self):
        tkmisc.center_window(self._body)
        tkmisc.dialog_effect(self._body)

    # ====================================
    #               PRIVATE
    # ====================================
    def _set_header(self, parent):
        # == top area
        frame = tk.Frame(parent)
        frame.pack(fill=tk.X)
        # label 'target project'
        label_target_project = tk.Label(frame, name="label_project",
                                        text="Project:")
        label_target_project.pack(side=tk.LEFT)
        # entry
        entry_path = tk.Entry(frame, name="entry_project",
                              state="readonly",
                              textvariable=self._strvar_target)
        entry_path.pack(side=tk.LEFT, expand=1, fill=tk.X)

    def _set_widgets_owner(self, parent):
        # frame
        frame = tk.Frame(parent)
        frame.grid(row=0, column=0, sticky="w")
        # label
        label_owner = tk.Label(frame, text="Owner")
        label_owner.pack(anchor="w")
        # entry
        entry_owner = tk.Entry(frame, width=20,
                               textvariable=self._strvar_owner,
                              state="readonly")
        entry_owner.pack(anchor="w")

    def _set_widgets_repo(self, parent):
        # frame
        frame = tk.Frame(parent)
        frame.grid(row=1, column=0, sticky="w")
        # label
        label_repo = tk.Label(frame, text="Repository *")
        label_repo.pack(anchor="w")
        # entry
        entry_repo = tk.Entry(frame, width=20,
                              textvariable=self._strvar_repo)
        entry_repo.pack(anchor="w")

    def _set_widgets_target_commitish(self, parent):
        # frame
        frame = tk.Frame(parent)
        frame.grid(row=2, column=0, sticky="w")
        # label
        label_target_commitish = tk.Label(frame, text="Target commitish")
        label_target_commitish.pack(anchor="w")
        # entry
        entry_target_commitish = tk.Entry(frame, width=20,
                                          textvariable=self._strvar_target_commitish)
        entry_target_commitish.pack(anchor="w")

    def _set_widgets_tag_name(self, parent):
        # frame
        frame = tk.Frame(parent)
        frame.grid(row=0, column=1, sticky="w")
        # label
        label_tag_name = tk.Label(frame, text="Tag name *")
        label_tag_name.pack(anchor="w")
        # entry
        entry_tag_name = tk.Entry(frame, width=20,
                                  textvariable=self._strvar_tag_name)
        entry_tag_name.pack(anchor="w")

    def _set_widgets_description(self, parent):
        # frame
        frame = tk.Frame(parent)
        frame.grid(row=3, column=0, columnspan=3, sticky="we")
        # label
        label_description = tk.Label(frame, text="Description")
        label_description.pack(anchor="w")
        # text
        self._text_description = tk.Text(frame, name="text_description",
                                         height=5, width=40)
        self._text_description.pack(anchor="w", fill=tk.X)

    def _set_widgets_release_name(self, parent):
        # frame
        frame = tk.Frame(parent)
        frame.grid(row=1, column=1, sticky="w")
        # label
        label_release_name = tk.Label(frame, text="Release name")
        label_release_name.pack(anchor="w")
        # entry
        entry_release_name = tk.Entry(frame, width=20,
                                      textvariable=self._strvar_release_name)
        entry_release_name.pack(anchor="w")

    def _set_widgets_archive_format(self, parent):
        # frame
        frame = tk.Frame(parent)
        frame.grid(row=2, column=1, sticky="w")
        # label
        label_archive_format = tk.Label(frame, text="Archive format")
        label_archive_format.pack(anchor="w")
        # entry
        strvar = tk.StringVar(value="zip")
        entry_archive_format = tk.Entry(frame, width=20,
                                        textvariable=strvar,
                                        state="readonly")
        entry_archive_format.pack(anchor="w")

    def _set_widgets_asset_name(self, parent):
        # frame
        frame = tk.Frame(parent)
        frame.grid(row=0, column=2, sticky="w")
        # label
        label_asset_name = tk.Label(frame, text="Asset name")
        label_asset_name.pack(anchor="w")
        # entry
        entry_asset_name = tk.Entry(frame, width=20,
                                    textvariable=self._strvar_asset_name)
        entry_asset_name.pack(anchor="w")

    def _set_widgets_asset_label(self, parent):
        # frame
        frame = tk.Frame(parent)
        frame.grid(row=1, column=2, sticky="w")
        # label
        label_asset_label = tk.Label(frame, text="Asset label")
        label_asset_label.pack(anchor="w")
        # entry
        entry_asset_label = tk.Entry(frame, width=20,
                                     textvariable=self._strvar_asset_label)
        entry_asset_label.pack(anchor="w")

    def _set_widgets_describe_mandatory_fields(self, parent):
        label = tk.Label(parent, name="label_mandatory",
                         text="* Mandatory fields")
        label.grid(row=4, column=0, sticky="w", pady=10)

    def _set_widgets_footer(self, parent):
        frame_footer = tk.Frame(parent)
        frame_footer.pack(fill=tk.X, padx=2, pady=2)
        button_publishing = tk.Button(frame_footer, name="button_confirm",
                                    text="Publish",
                                    command=self._on_click_publishing)
        button_publishing.pack(side=tk.RIGHT)
        button_cancel = tk.Button(frame_footer, name="button_cancel",
                                  text="Cancel",
                                  command=self._on_click_cancel)
        button_cancel.pack(side=tk.RIGHT, padx=(20, 2))
        # checkbutton Draft
        checkbutton_run_scripts = tk.Checkbutton(frame_footer, text="Draft",
                                               variable=self._intvar_draft)
        checkbutton_run_scripts.pack(side=tk.LEFT, padx=(0, 10))
        # checkbutton pre-release
        checkbutton_prerelease = tk.Checkbutton(frame_footer, text="Pre-release",
                                                variable=self._intvar_prerelease)
        checkbutton_prerelease.pack(side=tk.LEFT)

    def _on_click_cancel(self):
        self.destroy()

    def _on_click_publishing(self):
        if not self._check_mandatory_field():
            return
        confirm = Confirm(self._body, title="Confirmation",
                          header="Ready to publish your project",
                          message="Do you want to continue ?")
        confirm.build_wait()
        if not confirm.ok:
            return
        owner = self._strvar_owner.get()
        repo = self._strvar_repo.get()
        name = self._strvar_release_name.get()
        tag_name = self._strvar_tag_name.get()
        target_commitish = self._strvar_target_commitish.get()
        description = self._text_description.get("1.0","end-1c")
        prerelease = True if self._intvar_prerelease.get() == 1 else False
        draft = True if self._intvar_draft.get() == 1 else False
        asset_name = self._strvar_asset_name.get()
        asset_label = self._strvar_asset_label.get()
        self._asset_path = os.path.join(self._target_project,
                                        "pyrustic_data", "dist",
                                        "{}.zip".format(self._asset_version))
        # threadom stuff
        threadom = self._main_view.threadom
        host = self._main_host.publishing
        host_args = (owner, repo, name, tag_name, target_commitish,
                     description, prerelease, draft,
                     self._asset_path, asset_name, asset_label)
        consumer = self._notify_publishing_response
        threadom.run(host, args=host_args, consumer=consumer)
        self._cache_toast_processing = Toast(self._body, message="Processing...",
                                             duration=None)
        self._cache_toast_processing.build()

    def _check_mandatory_field(self):
        if self._strvar_repo.get() == "":
            toast = Toast(self._body, message="Please set the repository name")
            toast.build()
            return False
        if self._strvar_tag_name.get() == "":
            toast = Toast(self._body, message="Please set the tag name")
            toast.build()
            return False
        return True

    def _populate(self):
        self._target_project = self._main_host.target_project()
        data = self._main_host.about_target_project()
        target = data["target"]
        project_name = data["project_name"]
        version = self._asset_version
        owner = self._main_host.login
        default_target_commitish = "master"
        # target
        self._strvar_target.set(target)
        # owner
        self._strvar_owner.set(owner)
        # repo
        self._strvar_repo.set(project_name)
        # target commitish
        self._strvar_target_commitish.set(default_target_commitish)
        # tag name
        self._strvar_tag_name.set("v{}".format(version))
        # release name
        self._strvar_release_name.set("v{}".format(version))
        # asset name
        cache = "{}-v{}-released-by-{}.zip".format(project_name, version, owner)
        self._strvar_asset_name.set(cache)
        # asset label
        self._strvar_asset_label.set("Download the release")
        # description
        owner = self._strvar_owner.get()
        repo = self._strvar_repo.get()
        pyrustic_link = "https://github.com/pyrustic/pyrustic#readme"
        hubstore_link = "https://github.com/pyrustic/hubstore#readme"
        cache = "Install [Hubstore]({}):\n\n".format(hubstore_link)
        cache += "```bash\n"
        cache += "$ pip install hubstore\n"
        cache += "```\n\n"
        cache += "Run Hubstore:\n\n"
        cache += "```bash\n"
        cache += "$ hubstore\n"
        cache += "```\n\n"
        cache += "Then paste/type this in Hubstore:\n`{}/{}`\n\n".format(owner, repo)
        cache += "Enjoy !\n\n\n"
        cache += "Packaged and released with [Pyrustic]({}).".format(pyrustic_link)
        self._text_description.insert("1.0", cache)

    def _notify_publishing_response(self, data):
        if self._cache_toast_processing:
            self._cache_toast_processing.destroy()
            self._cache_toast_processing = None
        meta_code = data["meta_code"]
        status_code = data["status_code"]
        status_text = data["status_text"]
        message = "Successfully published"
        messages = {0: "Successfully published !",
                    1: "Failed to cache the target project\n{}".format(data["data"]),
                    2: "Tests failed\n{}".format(data["data"]),
                    3: "Failed to execute prolog\n{}".format(data["data"]),
                    4: "Failed to zip\n{}".format(data["data"]),
                    5: "Failed to create release\n{}".format(status_text),
                    6: "Failed to upload zip\n{}".format(status_text),
                    7: "Failed to execute epilog\n{}".format(data["data"])}
        message = messages[meta_code]
        if meta_code == 0:
            toast = Toast(self._body, message=message)
            toast.build_wait()
            data = (self._strvar_owner.get(), self._strvar_repo.get())
            self._main_view.central_view.add_node(*data)
            self.destroy()
        else:
            failure_view = FailureView(self._body, self._main_view,
                                       self._main_host, message)
            failure_view.build_wait()



import tkinter as tk
from pyrustic.widget.tree import Tree
from hubway.view.node_view import NodeView


class TreeView(Tree):
    def __init__(self, master, main_view, main_host):
        super().__init__(master, spacing=15)
        self._main_view = main_view
        self._main_host = main_host
        self._cache = {}

    # =============================
    #           PUBLIC
    # =============================


    # =============================
    #      IMPLEMENTATION OF TREE
    # =============================
    def _on_display(self):
        # insert first node
        node_id = self.insert()
        self.ghost(node_id)
        # request last activity
        host = self._main_host.last_activity
        consumer = (lambda data, self=self:
                    self.feed(datatype="last_activity", data=data))
        self._main_view.threadom.run(host, consumer=consumer)

    def _on_build_node(self, frame, node):
        node_id = node["node_id"]
        if node_id == 0:
            return
        node_view = NodeView(frame, self, self._main_view, self._main_host, node_id)
        node_view.build_pack(expand=1, fill=tk.X, side=tk.LEFT)
        self.tag(node_id, data={"node_view": node_view})
        node_type = node["data"]["type"]
        if node_type in ("description", "latest_release", "total_downloads"):
            node_view.populate()

    def _on_feed(self, *args, **kwargs):
        datatype = kwargs.get("datatype")
        data = kwargs.get("data")
        if datatype == "last_activity":
            self._load_last_activity(data)
        elif datatype == "add_owner_repo":
            self._add_owner_repo(*data)

    def _on_expand(self, node):
        node_id = node["node_id"]
        node_type = node["data"]["type"]
        node_view = node["data"].get("node_view", None)
        if node_view:
            node_view.edit_state(expanded=True)
        if node_type == "repo" and not self.children(node_id):
            self._insert_repo_sub_nodes(node_id)

    def _on_collapse(self, node):
        node_view = node["data"].get("node_view", None)
        if node_view:
            node_view.edit_state(expanded=False)

    # =============================
    #           PRIVATE
    # =============================
    def _load_last_activity(self, data):
        for owner, repos in data.items():
            # add owner to tree
            data = {"type": "owner", "name": owner}
            node_id = self.insert(parent=0, data=data, expand=True)
            for repo in repos:
                # add repo to owner
                data = {"type": "repo", "name": repo}
                self.insert(parent=node_id, data=data)

    # ===================
    def _request_data(self, node, node_type, node_view):
        repo_node = self.node(node["parent"])
        owner_node = self.node(repo_node["parent"])
        repo = repo_node["data"]["name"]
        owner = owner_node["data"]["name"]
        threadom = self._main_view.threadom
        host_choice = {"description": self._main_host.repo_description,
                       "latest_release": self._main_host.latest_release,
                       "total_downloads": self._main_host.latest_releases_downloads}
        host = host_choice[node_type]
        host_args = (owner, repo)
        consumer = (lambda data, datatype=node_type,
                           node_view=node_view:
                    node_view.feed(datatype, data=data))
        threadom.run(host, args=host_args, consumer=consumer)

    def _add_owner_repo(self, owner, repo):
        owner_node_id = self._add_owner(owner)
        repo_node_id = self._add_repo(owner_node_id, repo)
        # move owner node to top
        self.move(owner_node_id)
        # expand owner node
        self.expand(owner_node_id)
        # pull scrollbar to top
        self._main_view.central_view.scrollbox.yview_moveto(0)
        # expand repo
        self.expand(repo_node_id)

    def _add_owner(self, owner):
        node_id = None
        for node in self.children(0):
            cache = node["data"]["name"]
            if cache.lower() == owner.lower():
                node_id = node["node_id"]
                return node_id
        # add owner to tree
        data = {"type": "owner", "name": owner}
        node_id = self.insert(parent=0, data=data, expand=True, index=0)
        return node_id

    def _add_repo(self, parent_node_id, repo):
        for node in self.children(parent_node_id):
            cache = node["data"]["name"]
            if cache.lower() == repo.lower():
                node_id = node["node_id"]
                self.delete(node_id)
        # add repo
        data = {"type": "repo", "name": repo}
        node_id = self.insert(parent=parent_node_id, data=data, index=0)
        return node_id

    def _insert_repo_sub_nodes(self, node_id):
        # insert Description Node
        data = {"type": "description", "name": "Repository description"}
        self.insert(node_id, container=False, data=data)
        # insert Latest Release Node
        data = {"type": "latest_release", "name": "Latest release"}
        self.insert(node_id, container=False, data=data)
        # insert Total Downloads Node
        data = {"type": "total_downloads", "name": "Latest ten (pre)releases"}
        self.insert(node_id, container=False, data=data)



from pyrustic.gurl import dict_to_json_body
from pyrustic.manager.misc import funcs
import urllib.parse


class PublishingHost:
    def __init__(self, gurl, owner, repo):
        self._gurl = gurl
        self._owner = owner
        self._repo = repo

    def publishing(self, name, tag_name,
                   target_commitish, description,
                   prerelease, draft,
                   asset_path, asset_name, asset_label):
        """
        Return {"meta_code":, "status_code", "status_text", "data"}
        meta code:
            0- success
            1- failed to create release (check 'status_code', 'status_text')
            2- failed to upload zip (check 'status_code', 'status_text')
        """
        publishing_result = {"meta_code": None, "status_code": None,
                            "status_text": None, "data": None}
        # == create release
        response = self._create_release(self._owner, self._repo, name, tag_name,
                                        target_commitish, description,
                                        prerelease, draft)
        code = response.code
        if code != 201:
            publishing_result["meta_code"] = 5
            publishing_result["status_code"] = code
            publishing_result["status_text"] = response.status[1]
            return publishing_result
        # == upload asset
        upload_url = response.json["upload_url"]
        response = self._upload_asset(upload_url, asset_path,
                                      asset_name, asset_label)
        code = response.code
        if code != 201:
            publishing_result["meta_code"] = 6
            publishing_result["status_code"] = code
            publishing_result["status_text"] = response.status[1]
            return publishing_result
        publishing_result["meta_code"] = 0
        return publishing_result

    def _publishing(self, name, tag_name,
                 target_commitish, description, prerelease,
                 asset_path, asset_name, asset_label):
        # create release
        response = self._create_release(self._owner, self._repo, name, tag_name,
                                        target_commitish, description,
                                        prerelease)
        code = response.code
        if code == 201:
            # upload asset
            upload_url = response.json["upload_url"]
            response = self._upload_asset(upload_url, asset_path,
                                          asset_name, asset_label)
        return response.status

    def _create_release(self, owner, repo, name, tag_name, target_commitish,
                        description, prerelease, draft):
        res = "/repos/{}/{}/releases".format(owner, repo)
        body = {"tag_name": tag_name, "target_commitish": target_commitish,
                "name": name, "body": description, "draft": draft,
                "prerelease": prerelease}
        body = dict_to_json_body(body)
        response = self._gurl.request(funcs.get_hub_url(res),
                                      body=body, method="POST")
        return response

    def _upload_asset(self, upload_url, path, name, label):
        url = upload_url.replace("{?name,label}", "?{}")
        parameters = urllib.parse.urlencode({"label": label, "name": name})
        url = url.format(parameters)
        with open(path, "rb") as file:
            data = file.read()
        header = {"Content-Type": "application/zip"}
        response = self._gurl.request(url, body=data, method="POST",
                                      headers=header)
        return response



# NOTE TO MYSELF: DON'T IMPORT ANY MODULE FROM MY PACKAGE HERE !
import setuptools


with open("README.md", "r") as file:
    LONG_DESCRIPTION = file.read()


NAME = "hubway"
VERSION = "0.0.3"
AUTHOR = "Pyrustic Evangelist"
EMAIL = "pyrustic@protonmail.com"
DESCRIPTION = "A Pyrustic tool to publish Python desktop app to Github"
URL = "https://github.com/pyrustic/hubway"


setuptools.setup(
    name=NAME,
    version=VERSION,
    author=AUTHOR,
    author_email=EMAIL,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    url=URL,
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=["pyrustic"],
    entry_points={
        "console_scripts": [
            "hubway = hubway.main:main",
        ],
    },
    python_requires='>=3.5',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



"""
WARNING ! DON'T EDIT THIS FILE ! OTHERWISE YOU WILL LOSE YOUR SOUL !
Instead, edit the file "app.json" in the folder "pyrustic_data"

Are available:

    - ROOT_DIR        str     example: "/path/to/my/project"
    - PROJECT_NAME    str     example: "project"
    - PROJECT_TITLE   str     example: "Project"
    - VERSION         str     example: "0.0.1"
    - AUTHOR          str     example: "John Jr. Doe"
    - EMAIL           str     example: "homo@sapiens.earth"
    - DEV_MODE        bool    example: True

"""


import os.path
from pyrustic.jasonix import Jasonix


def _get_data():
    root_dir = os.path.dirname(os.path.abspath(__file__))
    about_json_path = os.path.join(root_dir, "pyrustic_data", "app.json")
    project_name = os.path.basename(root_dir)
    project_title = project_name.capitalize()
    dev_mode = True
    author = ""
    email = ""
    version = "0.0.1"
    if os.path.exists(about_json_path):
        jasonix = Jasonix(about_json_path, readonly=True)
        project_name = jasonix.data.get("project_name", project_name)
        project_title = jasonix.data.get("project_title", project_title)
        version = jasonix.data.get("version", version)
        author = jasonix.data.get("author", author)
        email = jasonix.data.get("email", email)
        dev_mode = jasonix.data.get("dev_mode", dev_mode)
    return {"root_dir": root_dir,
            "project_name": project_name,
            "project_title": project_title,
            "version": version,
            "author": author,
            "email": email,
            "dev_mode": dev_mode}


_data = _get_data()

ROOT_DIR = _data["root_dir"]
PROJECT_NAME = _data["project_name"]
PROJECT_TITLE = _data["project_title"]
VERSION = _data["version"]
AUTHOR = _data["author"]
DEV_MODE = _data["dev_mode"]



import tkinter as tk
from pyrustic.viewable import Viewable
from pyrustic import pymisc
from pyrustic.widget.toast import Toast


class HeaderView(Viewable):
    def __init__(self, master, main_view, main_host):
        self._master = master
        self._main_view = main_view
        self._main_host = main_host
        self._body = None
        self._toast_loading = None
        # stringvar
        self._stringvar = tk.StringVar()

    def show_rate(self, data):
        if self._toast_loading:
            self._toast_loading.destroy()
            self._toast_loading = None
        status_code, status_text, data = data
        message = None
        duration = 5000
        if status_code in (200, 304):
            message = "Rate Limit:\t{}\nRemaining :\t{}".format(data["limit"],
                                                             data["remaining"])
            message = pymisc.tab_to_space(message, tab_size=4)
        else:
            duration = 1000
            message = "Failed to load data\n{}".format(status_text)
        toast = Toast(self._body, message=message, duration=duration)
        toast.build()

    # =========================================
    #               LIFECYCLE
    # =========================================
    def _on_build(self):  # TODO, on windows, the ">" button isn't well aligned with widgets at Left
        self._body = tk.Frame(self._master)
        # label Query
        label_query = tk.Label(self._body, name="label_query", text="Query:")
        label_query.pack(side=tk.LEFT, anchor="n")
        # entry
        entry_search = tk.Entry(self._body, name="entry_search",
                                width=40,
                                textvariable=self._stringvar)
        entry_search.bind("<Return>", lambda e, self=self: self._on_click_search())
        entry_search.pack(side=tk.LEFT, anchor="n")
        entry_search.focus()
        # button run
        button_run = tk.Button(self._body, name="button_go", text=">",
                               command=self._on_click_search)
        button_run.pack(side=tk.LEFT, anchor="n")
        # button rate
        button_rate = tk.Button(self._body, name="button_rate", text="Rate",
                                command=self._on_click_rate)
        button_rate.pack(side=tk.RIGHT, anchor="n", padx=2, pady=2)

    def _on_display(self):
        pass

    def _on_destroy(self):
        pass

    # =========================================
    #               PRIVATE
    # =========================================
    def _on_click_search(self):
        data = self._stringvar.get()
        if data:
            data = self._main_host.extract_owner_repo_from_url(data)
        if not data:
            return
        self._stringvar.set("{}/{}".format(*data))
        self._main_view.central_view.add_node(*data)

    def _on_click_rate(self):
        if self._toast_loading:
            return
        self._toast_loading = Toast(self._body,
                                    message="Rate Limit: Loading...",
                                    duration=None)
        self._toast_loading.build()
        threadom = self._main_view.threadom
        host = self._main_host.rate
        consumer = self.show_rate
        threadom.run(host, consumer=consumer)



from pyrustic.theme import Theme
from pyrustic import default_style
from pyrustic.theme.cyberpunk import Cyberpunk
from pyrustic.theme.cyberpunk import constant
from pyrustic.theme.cyberpunk.pyrustic_widget import tree
from pyrustic.theme.cyberpunk.native_widget import label
from pyrustic.theme.cyberpunk.native_widget import entry
from pyrustic.theme.cyberpunk.native_widget import button
from pyrustic.theme.cyberpunk.native_widget import text
from pyrustic.theme.cyberpunk.native_widget import checkbutton
from pyrustic.theme.cyberpunk.native_widget import toplevel


# ========================================
# HUB THEME BASED ON CYBERPUNK THEME
# ========================================
def get_theme():
    theme = Cyberpunk()
    theme.add_theme(_get_general_theme())
    return theme


# ===================================
#              GENERAL
# ===================================
def _get_general_theme():
    theme = Theme()
    theme.add_style(_get_label_query_style(), scope="*label_query*")
    theme.add_style(_get_button_go_style(), scope="*button_go*")
    theme.add_style(_get_button_rate_style(), scope="*button_rate*")
    theme.add_style(_get_button_publishing_style(), scope="*button_publishing*")
    theme.add_style(_get_button_expander_style(), scope="*button_expander*")
    theme.add_style(_get_toolbar_button_close_style(), scope="*button_close*")
    theme.add_style(_get_checkbutton_auth_style(), scope="*checkbutton_auth*")
    theme.add_style(_get_label_owner_style(), scope="*label_owner*")
    theme.add_style(_get_label_repo_style(), scope="*label_repo*")
    theme.add_style(_get_button_refresh_style(), scope="*button_refresh*")
    theme.add_style(_get_button_retry_style(), scope="*button_retry*")
    theme.add_style(_get_label_info_title_style(), scope="*label_info_title*")
    theme.add_style(_get_label_error_style(), scope="*label_error*")
    theme.add_style(_get_label_counts_style(), scope="*label_counts*")
    theme.add_style(_get_button_cancel_style(), scope="*button_cancel*")
    theme.add_style(_get_button_confirm_style(), scope="*button_confirm*")
    theme.add_style(_get_entry_style(), scope="*Entry*")
    theme.add_style(_get_entry_search_style(), scope="*entry_search*")
    theme.add_style(_get_entry_repo_description_style(),
                    scope="*entry_repo_description*")
    theme.add_style(_get_toplevel_style(), scope="*Toplevel.")
    theme.add_style(_get_text_description_style(), scope="*text_description*")
    theme.add_style(_get_label_repo_style(), scope="*frame_form*Label*")
    theme.add_style(_get_label_project_style(), scope="*label_project*")
    theme.add_style(_get_entry_project_style(), scope="*entry_project*")
    theme.add_style(label.get_style(), scope="*label_mandatory*")
    theme.add_style(_get_failure_view_text_style(), scope="*failure_view*")
    return theme

# label query
def _get_label_query_style():
    style = label.get_style()
    style.background = "#005954"
    style.foreground = "#ECFFFF"
    style.font = constant.FONT_FAV_BOLD
    return style

# button go (search)
def _get_button_go_style():
    style = button.get_style()
    style.highlightBackground = "#000000"
    style.highlightColor = "#000000"
    style.background = "#ffffff"
    style.foreground = "#005954"
    style.activeBackground = "#005954"
    style.activeForeground = "#ECFFFF"
    style.highlightThickness = 0
    style.relief = "flat"
    style.padY = 0
    return style

# button rate
def _get_button_rate_style():
    style = button.get_style()
    style.background = "#181818"
    style.foreground = "#A098A0"
    style.highlightBackground = "#484048"
    style.highlightColor = "white"
    style.highlightThickness = 1
    style.relief = "flat"
    style.padY = 0
    return style

# button publishing
def _get_button_publishing_style():
    style = button.get_style()
    style.background = "#285B28"
    style.foreground = "#E0FFFF"
    style.highlightBackground = "#689B68"
    style.highlightColor = "white"
    style.highlightThickness = 1
    style.relief = "flat"
    return style

# expander button
def _get_button_expander_style():
    style = default_style.Button()
    style.font = constant.FONT_FAV_BOLD
    style.background = constant.COLOR_BLACK
    style.foreground = "gray"
    style.highlightThickness = 0
    style.borderWidth = 0
    style.activeBackground = "#F0F8FF"
    style.padX = 3
    style.padY = 1
    return style

# button close
def _get_toolbar_button_close_style():
    style = default_style.Button()
    style.background = "#191919"
    style.activeBackground = "#670000"
    style.activeForeground = "#FF0023"
    style.foreground = "#606060"
    style.borderWidth = 0
    style.highlightThickness = 0
    style.padX = 3
    style.padY = 0
    return style

# checkbox 'Authentication'
def _get_checkbutton_auth_style():
    style = checkbutton.get_style()
    style.foreground = "#D0D0D0"
    style.background = "#001B1B"
    style.activeBackground = "#003333"
    return style

# label owner
def _get_label_owner_style():
    style = label.get_style()
    style.foreground = "#9F9F9F"
    return style

# label owner
def _get_label_repo_style():
    style = label.get_style()
    style.foreground = "#D0D0D0"
    style.font = constant.FONT_FAV_BOLD
    return style

# button refresh
def _get_button_refresh_style():
    style = _get_button_rate_style()
    return style

# button retry
def _get_button_retry_style():
    style = button.get_style()
    style.background = "#301818"
    style.foreground = "#B898A0"
    style.highlightBackground = "#604048"
    style.highlightColor = "white"
    style.highlightThickness = 1
    style.relief = "flat"
    style.padY = 0
    return style

# node info
def _get_node_info_style():
    style = default_style.Frame()
    style.background = "#002323"
    return style

# label info title
def _get_label_info_title_style():
    style = label.get_style()
    style.foreground = "#9F9FC7"
    return style

# label error
def _get_label_error_style():
    style = label.get_style()
    style.foreground = "#D898A0"
    return style

# label stargazers
def _get_label_counts_style():
    style = label.get_style()
    style.font = constant.FONT_FAV_BOLD
    return style

# entry repo description
def _get_entry_repo_description_style():
    style = default_style.Entry()
    style.readonlyBackground = constant.COLOR_BLACK
    style.background = constant.COLOR_BLACK
    style.font = constant.FONT_FAV_NORMAL
    style.foreground = "#CFCFCF"
    return style

# button cancel
def _get_button_cancel_style():
    style = _get_button_retry_style()
    return style

# button confirm
def _get_button_confirm_style():
    style = _get_button_publishing_style()
    style.padY = 0
    return style

# entries
def _get_entry_style():
    style = entry.get_style()
    style.background = "#182020"
    style.foreground = "#C8D8E0"
    style.insertBackground = "#C8D8E0"
    style.readonlyBackground = "#101818"
    return style

# entry search
def _get_entry_search_style():
    style = entry.get_style()
    style.background = "white"
    style.insertBackground = constant.COLOR_BLACK
    return style

# toplevel
def _get_toplevel_style():
    style = toplevel.get_style()
    return style

# label project
def _get_label_project_style():
    style = _get_label_query_style()
    return style

# text description
def _get_text_description_style():
    style = text.get_style()
    style.highlightThickness = 0
    style.insertBackground = "#C8D8E0"
    style.background = "#182020"
    style.foreground = "#C8D8E0"
    return style

# entry project
def _get_entry_project_style():
    style = entry.get_style()
    return style

def _get_failure_view_text_style():
    style = text.get_style()
    style.foreground = "#C8C8C8"
    style.highlightThickness = 0
    style.background = constant.COLOR_BLACK
    return style
